---------------------------------------------------------------------
--  PhaseX - Copyright 2015 Ranfan@Blackrock-EU (candy@taomi.net)  --
-- All rights reserved. Unauthorized redistribution is prohibited. --
---------------------------------------------------------------------

PhaseX_Frame = {}

local config = PhaseX_Config
local frame = PhaseX_Frame

-- Background Frame
local phaseXBackground = CreateFrame("Frame", "PhaseXBackground", UIParent)
phaseXBackground:SetHeight(config.BarHeight)
phaseXBackground:SetWidth(config.BarWidth)
phaseXBackground:SetPoint(config.BarPoint, UIParent, config.BarRelativePoint, config.BarOffsetX, config.BarOffsetY)
phaseXBackground:SetBackdrop({
	bgFile = config.BackgroundTexture, 
	edgeFile = config.BackgroundTexture, 
	tile = false, tileSize = 0, edgeSize = 1, 
	insets = { left = 0, right = 0, top = 0, bottom = 0}
})
phaseXBackground:SetBackdropColor(unpack(config.PhaseColors[1]))
phaseXBackground:SetBackdropBorderColor(.2, .2, .2)
phaseXBackground:SetMovable(true)
phaseXBackground:SetClampedToScreen(true)
phaseXBackground:SetUserPlaced(true)
frame.Background = phaseXBackground

-- Status Frame
local phaseXBar = CreateFrame("StatusBar", "PhaseXBar", phaseXBackground)
phaseXBar:SetStatusBarTexture(config.BarTexture)
phaseXBar:SetMinMaxValues(0, 10000)
phaseXBar:SetPoint("TOPLEFT", phaseXBackground, "TOPLEFT", 2, -2)
phaseXBar:SetPoint("BOTTOMRIGHT", phaseXBackground, "BOTTOMRIGHT", -2, 2)
phaseXBar:SetStatusBarColor(0, 1, 0)
frame.Bar = phaseXBar

-- Status Text Center
local phaseXTextCenter = PhaseXBar:CreateFontString(nil, "OVERLAY")
phaseXTextCenter:SetFont(config.TextFont, config.TextSize, config.TextFlags)
phaseXTextCenter:SetPoint("CENTER", phaseXBackground, "CENTER")
frame.TextCenter = phaseXTextCenter

-- Status Text Left
local phaseXTextLeft = PhaseXBar:CreateFontString(nil, "OVERLAY")
phaseXTextLeft:SetFont(config.TextFont, config.TextSize, config.TextFlags)
phaseXTextLeft:SetPoint("LEFT", phaseXBackground, "LEFT")
frame.TextLeft = phaseXTextLeft

-- Status Text Right
local phaseXTextRight = PhaseXBar:CreateFontString(nil, "OVERLAY")
phaseXTextRight:SetFont(config.TextFont, config.TextSize, config.TextFlags)
phaseXTextRight:SetPoint("RIGHT", phaseXBackground, "RIGHT")
frame.TextRight = phaseXTextRight

frame.Active = false

frame.LastUpdate = 0

frame.GetNpcId = function(unitId)
	local guid = UnitGUID(unitId);
	if (guid == nil) then
		return 0
	end
	local type, zero, server_id, instance_id, zone_uid, npc_id, spawn_uid = strsplit("-",guid);
	if (type == "Creature" or type == "Vehicle" or type == "GameObject" or type == "Pet") then
		return tonumber(npc_id)
	end
	return 0
end

frame.GetBossList = function()
	local list = {}
	for i = 1, MAX_BOSS_FRAMES do
		list[frame.GetNpcId("boss"..i)] = "boss"..i
	end
	list[0] = nil
	return list
end

frame.GetAuraList = function(unitId)
	local list = {}
	if (UnitExists(unitId)) then
		for i = 1, 40 do
			local _, _, _, _, _, _, _, _, _, _, spellId = UnitAura(unitId, i)
			if (spellId ~= nil) then
				list[spellId] = unitId
			end
		end
	end
	return list
end

frame.GetPercentValue = function(currentValue, maxValue)
	if (currentValue == nil) then
		currentValue = 0
	end
	if (maxValue == nil or maxValue == 0) then
		maxValue = 1
	end
	local percentValue = currentValue / maxValue
	if (percentValue < 0) then
		percentValue = 0
	end
	if (percentValue > 1) then
		percentValue = 1
	end
	return percentValue
end

frame.GetRemainingTime = function(percentValue, timeStart)
	local currentDuration = GetTime() - timeStart
	if (percentValue == 1) then
		return ""
	end
	local remainingDuration = currentDuration * percentValue / (1 - percentValue)
	return format("%.2f", remainingDuration).."s"
end

frame.UpdateBar = function(percentValue, phaseCurrent, phaseMax, phaseStart)
	if (percentValue < 0) then
		percentValue = 0
	end
	if (percentValue > 1) then
		percentValue = 1
	end
	local phaseOffsetBackground = phaseMax - phaseCurrent + 1
	local phaseOffsetBar = phaseOffsetBackground + 1
	while (phaseOffsetBackground > #config.PhaseColors) do
		phaseOffsetBackground = phaseOffsetBackground - #config.PhaseColors + 1
	end
	while (phaseOffsetBar > #config.PhaseColors) do
		phaseOffsetBar = phaseOffsetBar - #config.PhaseColors + 1
	end
	frame.Background:SetBackdropColor(unpack(config.PhaseColors[phaseOffsetBackground]))
	frame.Bar:SetStatusBarColor(unpack(config.PhaseColors[phaseOffsetBar]))
	frame.Bar:SetValue(percentValue * 10000)
	
	local phaseText = ""
	if (phaseMax > 1) then
		phaseText = " Phase "..phaseCurrent
	end
	frame.TextLeft:SetText(phaseText)
	frame.TextCenter:SetText(format("%.2f", percentValue * 100).."%")
	frame.TextRight:SetText(frame.GetRemainingTime(percentValue, phaseStart))
end

frame.EncounterId = 0
frame.EncounterUnit = "target"
frame.PercentValue = 0
frame.PhaseCurrent = 1
frame.PhaseMax = 1
frame.PhaseStart = GetTime()

frame.Setup_Default = function()
	frame.EncounterId = 0
	frame.EncounterUnit = "target"
	frame.PercentValue = 1
	frame.PhaseCurrent = 1
	frame.PhaseMax = 1
--	frame.PhaseMax = 10
	frame.PhaseStart = GetTime()
	frame.UpdateBoss = function() end
	frame.UpdateEvent = function() end
	frame.UpdateStatus = function()
		frame.PercentValue = frame.GetPercentValue(UnitHealth(frame.EncounterUnit), UnitHealthMax(frame.EncounterUnit))
--		local state = floor(frame.PercentValue*10)
--		frame.UpdatePhase(10-state)
--		frame.PercentValue = (frame.PercentValue - (state/10))*10
	end
end

frame.SetupBoss = function()
	frame.Setup_Default()
end
frame.UpdateBoss = function() end
frame.UpdateEvent = function(event, ...) end
frame.UpdateStatus = function() end

frame.UpdatePhase = function(phase)
	if (phase > frame.PhaseMax) then
		phase = frame.PhaseMax
	end
	if (phase < 1) then
		phase = 1
	end
	if (frame.PhaseCurrent ~= phase) then
		frame.PhaseCurrent = phase
		frame.PhaseStart = GetTime()
		frame.PercentValue = 1
	end
end

frame.Enable = function()
	frame.LastUpdate = config.UpdateInterval
	frame.Background:Show()
	frame.Active = true
end

frame.Disable = function()
	frame.Active = false
	frame.EncounterId = 0
	frame.UpdateBoss = function() end
	frame.UpdateEvent = function(event, ...) end
	frame.UpdateStatus = function() end
	frame.Background:Hide()
end

frame.InFight = false

-- Show & Hide Events
frame.Background:RegisterEvent("ADDON_LOADED")
frame.Background:RegisterEvent("PLAYER_LOGIN")
frame.Background:RegisterEvent("PLAYER_ENTERING_WORLD")
frame.Background:RegisterEvent("PLAYER_REGEN_DISABLED")
frame.Background:RegisterEvent("PLAYER_REGEN_ENABLED")
frame.Background:RegisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT")
frame.Background:SetScript("OnEvent", function(self, event, arg)
	if (frame.Unlocked == true) then
		return
	end
	if (event == "PLAYER_REGEN_DISABLED") then
		frame.InFight = true
	end
	if (event == "PLAYER_REGEN_ENABLED") then
		frame.InFight = false
		if (UnitExists("boss1") or UnitExists("boss2") or UnitExists("boss3") or UnitExists("boss4") or UnitExists("boss5")) then
		else
			frame.Disable()
		end
	end
	if (event == "INSTANCE_ENCOUNTER_ENGAGE_UNIT") then
		if (frame.EncounterId == 0) then
			frame.SetupBoss()
			if (frame.EncounterId ~= 0) then
				frame.Enable()
			end
		else
			frame.UpdateBoss()
		end
		if (UnitExists("boss1") or UnitExists("boss2") or UnitExists("boss3") or UnitExists("boss4") or UnitExists("boss5")) then
		else
			if (frame.InFight == false) then
				frame.Disable()
			end
		end
	end
	local bossFightsOnly = true
	if (PhaseX_PlayerConfig ~= nil) then
		bossFightsOnly = PhaseX_PlayerConfig.BossFightsOnly
	end
	if (frame.EncounterId == 0 and bossFightsOnly ~= true) then
		if (event == "PLAYER_REGEN_DISABLED") then
			frame.Setup_Default()
			frame.Enable()
		end
		if (event == "PLAYER_REGEN_ENABLED") then
			frame.Disable()
		end
	end
	if (event == "PLAYER_ENTERING_WORLD") then
		frame.Disable()
	end
	if (event == "PLAYER_LOGIN") then
		print("|cFF00FF00PhaseX|cFFFFFFFF (|cFFFFA000"..GetAddOnMetadata("PhaseX","Version").."|cFFFFFFFF) loaded.")
		print("|cFFFFFFFF    Use |cFF00FF00/phasex|cFFFFFFFF for a list of available commands.")
	end
	if (event == "ADDON_LOADED" and arg == "PhaseX") then
		if (PhaseX_PlayerConfig == nil or #PhaseX_PlayerConfig < 8) then
			PhaseX_PlayerConfig = {}
			frame.ResetConfiguration(PhaseX_Config)
		else
			frame.ResetConfiguration(PhaseX_PlayerConfig)
		end
	end
end)

-- Tracking Event
frame.Bar:RegisterEvent("UNIT_SPELLCAST_START")
frame.Bar:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
frame.Bar:RegisterEvent("UNIT_SPELLCAST_FAILED")
frame.Bar:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
frame.Bar:RegisterEvent("UNIT_AURA")
frame.Bar:SetScript("OnEvent", function(self, event, ...)
	local arg = {...}
	frame.UpdateEvent(event, unpack(arg))
end)

frame.Bar:SetScript("OnUpdate", function(self, elapsed)
	if (frame.Active == false) then
		return
	end

	frame.LastUpdate = frame.LastUpdate + elapsed
	
	if (frame.LastUpdate >= config.UpdateInterval) then
		frame.UpdateStatus()
		frame.UpdateBar(frame.PercentValue, frame.PhaseCurrent, frame.PhaseMax, frame.PhaseStart)
	
		frame.LastUpdate = frame.LastUpdate - config.UpdateInterval
	end
end)

frame.SetConfigurationBossFightsOnly = function(enabled)
	PhaseX_PlayerConfig.BossFightsOnly = enabled
end

frame.SetConfigurationPosition = function(positionpoint, positionrelative, positionx, positiony)
	PhaseX_PlayerConfig.BarOffsetX = positionx
	PhaseX_PlayerConfig.BarOffsetY = positiony
	PhaseX_PlayerConfig.BarRelativePoint = positionrelative
	PhaseX_PlayerConfig.BarPoint = positionpoint
	frame.Background:SetPoint(positionpoint, UIParent, positionrelative, positionx, positiony)
end

frame.SetConfigurationHeight = function(height)
	PhaseX_PlayerConfig.BarHeight = height
	frame.Background:SetHeight(height)
end

frame.SetConfigurationWidth = function(width)
	PhaseX_PlayerConfig.BarWidth = width
	frame.Background:SetWidth(width)
end

frame.SetConfigurationTextSize = function(size)
	PhaseX_PlayerConfig.TextSize = size
	frame.TextCenter:SetFont(config.TextFont, size, config.TextFlags)
	frame.TextLeft:SetFont(config.TextFont, size, config.TextFlags)
	frame.TextRight:SetFont(config.TextFont, size, config.TextFlags)
end

frame.ResetConfiguration = function(configtable)
	frame.SetConfigurationBossFightsOnly(configtable.BossFightsOnly)
	frame.SetConfigurationPosition(configtable.BarPoint, configtable.BarRelativePoint, configtable.BarOffsetX, configtable.BarOffsetY)
	frame.SetConfigurationHeight(configtable.BarHeight)
	frame.SetConfigurationWidth(configtable.BarWidth)
	frame.SetConfigurationTextSize(configtable.TextSize)
end

frame.Unlocked = false
local function SlashCommandHandler(msg, editbox)
	local list = {}
	for token in gmatch(msg, "[^%s]+") do
		list[#list+1] = token
	end
	
	if (frame.Active == true) then
		print("|cFF00FF00PhaseX|cFFFFFFFF - Commands are disabled while the frame is active.")
		return
	end
	
	local token = list[1]
	if (token == "reset") then
		frame.ResetConfiguration(PhaseX_Config)
		print("|cFF00FF00PhaseX|cFFFFFFFF - Configuration reset to default values.")
	elseif (token == "boss") then
		if (PhaseX_PlayerConfig.BossFightsOnly == true) then
			frame.SetConfigurationBossFightsOnly(false)
			print("|cFF00FF00PhaseX|cFFFFFFFF - Frame now shows during all fights.")
		else
			frame.SetConfigurationBossFightsOnly(true)
			print("|cFF00FF00PhaseX|cFFFFFFFF - Frame now only shows during boss fights.")
		end
	elseif (token == "unlock") then
		if (frame.Unlocked == true) then
			print("|cFF00FF00PhaseX|cFFFFFFFF - Frame already is unlocked.")
			return
		end
		frame.Unlocked = true
		frame.Background:EnableMouse(true)
		frame.Background:RegisterForDrag("LeftButton")
		frame.Background:SetScript("OnDragStart", frame.Background.StartMoving)
		frame.Background:SetScript("OnDragStop", frame.Background.StopMovingOrSizing)
		frame.TextCenter:SetText("100.00%")
		frame.TextLeft:SetText("Phase X")
		frame.TextRight:SetText("999.99s")
		ToggleFrame(frame.Background)
		print("|cFF00FF00PhaseX|cFFFFFFFF - Frame now is unlocked.")
	elseif (token == "lock") then
		if (frame.Unlocked == false) then
			print("|cFF00FF00PhaseX|cFFFFFFFF - Frame already is locked.")
			return
		end
		frame.Unlocked = false
		frame.Background:EnableMouse(false)
		frame.TextCenter:SetText("")
		frame.TextLeft:SetText("")
		frame.TextRight:SetText("")
		ToggleFrame(frame.Background)
		local pospoint, _, posrelative, posx, posy = frame.Background:GetPoint()
		frame.SetConfigurationPosition(pospoint, posrelative, posx, posy)
		print("|cFF00FF00PhaseX|cFFFFFFFF - Frame now is locked.")
	elseif (token == "height") then
		local currentHeight = frame.Background:GetHeight()
		if (#list > 1) then
			frame.SetConfigurationHeight(list[2])
			print("|cFF00FF00PhaseX|cFFFFFFFF - Old Frame Height: |cFFFFA000"..floor(currentHeight+0.5).."|cFFFFFFFF - New Frame Height: |cFFFFA000"..floor(frame.Background:GetHeight()+0.5).."|cFFFFFFFF")
		else
			print("|cFF00FF00PhaseX|cFFFFFFFF - Current Frame Height: |cFFFFA000"..floor(currentHeight+0.5).."|cFFFFFFFF")
		end
	elseif (token == "width") then
		local currentWidth = frame.Background:GetWidth()
		if (#list > 1) then
			frame.SetConfigurationWidth(list[2])
			print("|cFF00FF00PhaseX|cFFFFFFFF - Old Frame Width: |cFFFFA000"..floor(currentWidth+0.5).."|cFFFFFFFF - New Frame Width: |cFFFFA000"..floor(frame.Background:GetWidth()+0.5).."|cFFFFFFFF")
		else
			print("|cFF00FF00PhaseX|cFFFFFFFF - Current Frame Width: |cFFFFA000"..floor(currentWidth+0.5).."|cFFFFFFFF")
		end
	elseif (token == "textsize") then
		local _, fontSize, _ = frame.TextCenter:GetFont()
		if (#list > 1) then
			frame.SetConfigurationTextSize(list[2])
			local _, fontSizeNew, _ = frame.TextCenter:GetFont()
			print("|cFF00FF00PhaseX|cFFFFFFFF - Old Text Size: |cFFFFA000"..floor(fontSize+0.5).."|cFFFFFFFF - New Text Size: |cFFFFA000"..floor(fontSize+0.5).."|cFFFFFFFF")
		else
			print("|cFF00FF00PhaseX|cFFFFFFFF - Current Text Size: |cFFFFA000"..floor(fontSize+0.5).."|cFFFFFFFF")
		end
	else
		print("|cFF00FF00PhaseX|cFFFFFFFF (|cFFFFA000"..GetAddOnMetadata("PhaseX","Version").."|cFFFFFFFF). Available commands:")
		print("    |cFF00FF00/phasex reset|cFFFFFFFF - Resets the configuration to default values.")
		print("    |cFF00FF00/phasex boss|cFFFFFFFF - Toggles if the frame should be shown for all fights or for boss fights only.")
		print("    |cFF00FF00/phasex unlock|cFFFFFFFF - Unlocks the PhaseX frame to move it around.")
		print("    |cFF00FF00/phasex lock|cFFFFFFFF - Locks the PhaseX frame in the current position.")
		print("    |cFF00FF00/phasex height |cFFFFA000<newsize>|cFFFFFFFF - Gets the current height of the PhaseX frame or sets a new height.")
		print("    |cFF00FF00/phasex width |cFFFFA000<newsize>|cFFFFFFFF - Gets the current width of the PhaseX frame or sets a new width.")
		print("    |cFF00FF00/phasex textsize |cFFFFA000<newsize>|cFFFFFFFF - Gets the current text size of the PhaseX frame or sets a new text size.")
	end
end

SLASH_PHASEX1 = "/phasex"
SlashCmdList["PHASEX"] = SlashCommandHandler